/// @file    plugins/s3/S3Connection.h
/// @brief   s3 plugin.
/// @author  Martin Philipp Hellmich <mhellmic@cern.ch>
#ifndef S3Connection_H
#define	S3Connection_H

#include <pthread.h>
#include <set>
#include <time.h>
#include <vector>

#include <dmlite/cpp/catalog.h>
#include <dmlite/cpp/dmlite.h>
#include <dmlite/cpp/inode.h>

#include <boost/thread/locks.hpp>
#include <boost/thread/mutex.hpp>

#include "s3driver.h"
#include "s3objects.pb.h"

namespace dmlite {

struct ReplicaMeta {
  Replica replica;
  time_t ingest_time;
  S3PoolDetails pool;

  bool operator <  (const ReplicaMeta&) const;
};

class S3Connection {
  static boost::mutex replicaSetMutex;  
  static boost::mutex stackMySqlMutex;  
  static boost::mutex setStackMutex;  
  static std::set<ReplicaMeta> pendingReplicas;
public:
  S3Connection(/*const std::string& host, unsigned int port,
               const std::string& s3AccessKeyID,
               const std::string& s3SecretAccessKey,*/
               const time_t replicaTimeout);
  ~S3Connection();

  void setStackInstance(StackInstance*) throw (DmException);

  Location getQueryString(/*std::string host, unsigned int port,*/
                          std::string bucket, std::string key,
                          time_t expirationDate, S3PoolDetails pool);
  
  Location putQueryString(/*std::string host, unsigned int port,*/
                          std::string bucket, std::string key,
                          time_t expirationDate, S3PoolDetails pool);
  
  S3RequestResponse removeReplica(Replica replica, S3PoolDetails pool);


  void startAvailabilityCheck() throw (DmException);
  void stopAvailabilityCheck();

  bool checkReplicaAvailability(Replica replica, S3PoolDetails pool);

  void addPendingReplica(const Replica& replica, S3PoolDetails pool);
  void setCheckTimeout(time_t timeout);

protected:
private:

  S3Driver     driver;
  pthread_t    checkThread;
  time_t       replicaTimeout;
//  std::string  host;
//  unsigned int port;

  StackInstance *stack;

  void *checkAvailabilityLoop();
  static void *checkAvailabilityLoopEntry(void *This);
};


};
#endif // S3connection_H
