/// @file    plugins/s3/S3Factory.h
/// @brief   plugin to store data in a s3 backend.
/// @author  Martin Philipp Hellmich <mhellmic@cern.ch>
#ifndef S3Factory_H
#define	S3Factory_H

#include <time.h>

#include <boost/regex.hpp>

#include <dmlite/cpp/dmlite.h>
#include <dmlite/cpp/utils/security.h>
#include <dmlite/cpp/utils/urls.h>
#include <dmlite/cpp/utils/poolcontainer.h>

#include "S3Connection.h"

namespace dmlite {

class S3ConnectionFactory: public PoolElementFactory<S3Connection*> {
public:
  S3ConnectionFactory(const std::string& host, unsigned int port);
  ~S3ConnectionFactory();

  S3Connection* create();
  void   destroy(S3Connection* conn);
  bool   isValid(S3Connection* conn);

  //void   setS3SecretAccessKey(const std::string& key);

  void   setS3ReplicaTimeout(std::string duration);

  /// The S3 host to connect to.
  //std::string host_;

  /// The port to connect to.
  //unsigned int port_;

  /// The Amazon Access ID.
  //std::string s3AccessKeyID_;
protected:
private:
  
  /// The Secret Access Key.
  //std::string s3SecretAccessKey_;

  /// The timeout for pending replicas.
  time_t s3ReplicaTimeout_;
};


class S3Factory: public PoolDriverFactory {
public:
  S3Factory() throw (DmException);
  ~S3Factory() throw (DmException);
  void configure(const std::string& key, const std::string& value) throw (DmException);

  std::string implementedPool() throw();
  PoolDriver* createPoolDriver() throw (DmException);

  S3Connection* getConnection() throw (DmException);

  /// Release an S3 connection. If the reference counts drops to 0,
  /// the connection will be released on the pool.
  void releaseConnection(S3Connection*) throw (DmException);
  
protected:

private:
  /// Connection factory.
  S3ConnectionFactory connectionFactory_;

  /// Connection pool.
  PoolContainer<S3Connection*> connectionPool_;
  
  /// Key used to keep only one connection per thread and factory
  pthread_key_t thread_s3_conn_;
};

};
#endif	// S3Factory_H
