/// @file    plugins/s3/S3.h
/// @brief   plugin to store data in a s3 backend.
/// @author  Martin Philipp Hellmich <mhellmic@cern.ch>
#ifndef S3Driver_H
#define	S3Driver_H

#include <iostream>
#include <map>
#include <openssl/hmac.h>
#include <sstream>

#include <neon/ne_socket.h>
#include <neon/ne_request.h>
#include <neon/ne_session.h>

#include <dmlite/common/errno.h>
#include <dmlite/cpp/pooldriver.h>

#include "s3objects.pb.h"

#define DATE_FORMAT "%a, %d %b %Y %H:%M:%S GMT"

namespace dmlite {

class S3Driver {
public:
  S3Driver();

  ~S3Driver();

  Location getQueryString(std::string method, /*std::string host,
                          unsigned int port,*/ std::string bucket,
                          std::string key, time_t expirationDate,
                          S3PoolDetails pool);

  S3RequestResponse removeObject(/*std::string host, unsigned int port,*/
                                 std::string bucket, std::string key,
                                         S3PoolDetails pool);

  S3RequestResponse headObject(/*std::string host, unsigned int port,*/
                              std::string bucket, std::string key,
                              S3PoolDetails pool);


  private:
  /// The neon session to use
  std::map<std::string, ne_session*> neon_session_map;

  std::string canonicalize(std::string method, std::string bucket, 
                           std::string key, std::map<std::string,
                           std::string> headerMap, std::string param);

  std::string getSignature(std::string method, std::string bucket, 
                           std::string key, std::map<std::string,
                           std::string> headerMap, std::string param,
                           S3PoolDetails pool);

  std::string s3TimeStringFromNow();

  S3Error getS3Error(ne_request *request);

  std::string getS3ErrorAsString(ne_request *request);

  S3RequestResponse requestObject(std::string httpRequest,/* std::string host,
                                  unsigned int port,*/ std::string bucket,
                                  std::string key, S3PoolDetails pool);

  std::pair<std::string, std::string> getS3HostAndPath(/*std::string host,*/
                                                       S3PoolDetails pool,
                                                       std::string bucket,
                                                       std::string key);

  std::string base64Encode(const unsigned char* aContent, size_t aContentSize, long& aBase64EncodedStringLength);

  ne_session *getNeonSession(std::string adjustedHost,/* unsigned int port,*/
                             S3PoolDetails pool);

  void destroyNeonSessionsInMap();
};

};

#endif // S3Driver_H
